"""Randomly samples data to bring all class frequencies into a range."""

import numpy as np
from sklearn.utils import check_X_y

from imblearn.base import SamplerMixin
from imblearn.utils import check_target_type, hash_X_y
from imblearn.under_sampling import RandomUnderSampler
from imblearn.over_sampling import RandomOverSampler


class MinMaxRandomSampler(SamplerMixin):
    """Random samples data to bring all class frequencies into a range.

    Parameters
    ----------
    min_freq : int
        The minimum frequency for a class after sampling. All classes with
        fewer samples are over-sampled to have this number of samples.
    max_freq : int
        The maximum frequency for a class after sampling. All classes with
        more samples are under-sampled to have this number of samples.
    """

    def __init__(self, min_freq, max_freq):
        self.min_freq = min_freq
        self.max_freq = max_freq
        self.ratio = max_freq / min_freq

    def fit(self, X, y):
        """Find the classes statistics before to perform sampling.

        Parameters
        ----------
        X : {array-like, sparse matrix}, shape (n_samples, n_features)
            Matrix containing the data which have to be sampled.
        y : array-like, shape (n_samples,)
            Corresponding label for each sample in X.

        Returns
        -------
        self : object,
            Return self.
        """
        X, y = check_X_y(X, y, accept_sparse=['csr', 'csc'], dtype=None)
        y = check_target_type(y)
        self.ratio_ = self.ratio
        self.X_hash_, self.y_hash_ = hash_X_y(X, y)
        labels = np.unique(y)
        counts = np.bincount(y)
        under_dict = {}
        over_dict = {}
        for lbl in labels:
            count = counts[lbl]
            if count < self.min_freq:
                under_dict[lbl] = count
                over_dict[lbl] = self.min_freq
            elif count > self.max_freq:
                under_dict[lbl] = self.max_freq
                over_dict[lbl] = self.max_freq
            else:
                under_dict[lbl] = count
                over_dict[lbl] = count
        self.under_sampler = RandomUnderSampler(ratio=under_dict)
        self.over_sampler = RandomOverSampler(ratio=over_dict)
        return self

    def _sample(self, X, y):
        """Resample the dataset.

        Parameters
        ----------
        X : {array-like, sparse matrix}, shape (n_samples, n_features)
            Matrix containing the data which have to be sampled.
        y : array-like, shape (n_samples,)
            Corresponding label for each sample in X.

        Returns
        -------
        X_resampled : {ndarray, sparse matrix}, shape \
        (n_samples_new, n_features)
            The array containing the resampled data.
        y_resampled : ndarray, shape (n_samples_new)
            The corresponding label of `X_resampled`
        """
        new_X, new_y = self.under_sampler.fit_sample(X, y)
        return self.over_sampler.fit_sample(new_X, new_y)



"""Testing MinMaxRandomSampler."""

import numpy as np
import pandas as pd
from pdutil.transform import x_y_by_col_lbl

from imbutil.combine import MinMaxRandomSampler

_DATA1 = [
    [23, 'Jo', 1],
    [33, 'Bo', 1],
    [10, 'Ki', 2],
    [82, 'Ro', 2],
    [70, 'Pi', 2],
    [72, 'Po', 2],
    [14, 'Li', 3],
    [81, 'Lo', 3],
    [73, 'Mi', 3],
    [79, 'Mo', 3],
    [44, 'Ma', 3],
    [78, 'Me', 3],
]


def _df1():
    return pd.DataFrame(
        data=_DATA1,
        columns=['age', 'name', 'label'],
    )


def test_basic_minmax_sample():
    print("testing MinMaxRandomSampler fit_sample...")
    df = _df1()
    sampler = MinMaxRandomSampler(min_freq=3, max_freq=4)
    X, y = x_y_by_col_lbl(df, 'label')
    new_X, new_y = sampler.fit_sample(X, y)
    print(new_X)
    print(new_y)
    label_counts = np.bincount(new_y)
    assert label_counts[1] == 3
    assert label_counts[2] == 4
    assert label_counts[3] == 4


if __name__ == '__main__':
    test_basic_minmax_sample()
